@@ -18,21 +18,28 @@
 #
 """
 Unit tests for the schooltool.timetable.model module.
-
-$Id: test_model.py 5022 2005-09-20 09:30:52Z mg $
 """
+
 import calendar
 import unittest
+import doctest
+import itertools
 from datetime import date, time, timedelta, datetime
 from pprint import pformat
-from pytz import UTC
 
+from pytz import UTC
 from zope.interface.verify import verifyObject
+from zope.traversing.interfaces import IPhysicallyLocatable
 from zope.app.testing.placelesssetup import PlacelessSetup
-from zope.app.testing import ztapi
-from schoolbell.app.rest.tests.utils import NiceDiffsMixin, EqualsSortedMixin
-from schooltool.tests.helpers import diff, sorted
-from schooltool.timetable.tests.test_timetable import TermStub
+from zope.app.testing import setup
+from zope.component import provideAdapter
+from zope.interface import implements
+
+from schooltool.testing.util import NiceDiffsMixin
+from schooltool.testing.util import diff
+from schooltool.term.tests.test_term import TermStub
+from schooltool.app.interfaces import ISchoolToolApplication
+from schooltool.app.interfaces import IApplicationPreferences
 
 
 class TimetablePhysicallyLocatableAdapterStub:
@@ -58,31 +65,112 @@ class BaseTestTimetableModel:
         return result
 
 
+def createSimpleTimetable():
+    """Create a simple timetable.
+
+          Period | Day A    Day B
+          ------ : -------  ---------
+          Green  : English  Biology
+          Blue   : Math     Geography
+
+    """
+    from schooltool.timetable import Timetable, TimetableDay
+    from schooltool.timetable import TimetableActivity
+    tt = Timetable(('A', 'B'))
+    periods = ('Green', 'Blue')
+    tt["A"] = TimetableDay(periods)
+    tt["B"] = TimetableDay(periods)
+    tt["A"].add("Green", TimetableActivity("English"))
+    tt["A"].add("Blue", TimetableActivity("Math"))
+    tt["B"].add("Green", TimetableActivity("Biology"))
+    tt["B"].add("Blue", TimetableActivity("Geography"))
+    return tt
+
+
 class SequentialTestSetupMixin:
+    createTimetable = staticmethod(createSimpleTimetable)
 
-    def createTimetable(self):
-        """Create a simple timetable.
 
-              Period | Day A    Day B
-              ------ : -------  ---------
-              Green  : English  Biology
-              Blue   : Math     Geography
+class ApplicationStub(object):
+    implements(ISchoolToolApplication, IApplicationPreferences)
+    def __init__(self, timezone='UTC'):
+        self.timezone=timezone
+    def __call__(self, context):
+        return self
 
-        """
-        from schooltool.timetable import Timetable, TimetableDay
-        from schooltool.timetable import TimetableActivity
-        #from schooltool.component import getOptions
-        tt = Timetable(('A', 'B'))
-        #setPath(tt, '/path/to/tt')
-        #getOptions(tt).timetable_privacy = 'public'
-        periods = ('Green', 'Blue')
-        tt["A"] = TimetableDay(periods)
-        tt["B"] = TimetableDay(periods)
-        tt["A"].add("Green", TimetableActivity("English"))
-        tt["A"].add("Blue", TimetableActivity("Math"))
-        tt["B"].add("Green", TimetableActivity("Biology"))
-        tt["B"].add("Blue", TimetableActivity("Geography"))
-        return tt
+
+def doctest_BaseTimetableModel_createCalendar():
+    """Tests for BaseTimetableModel.createCalendar
+
+    We'll have to provide application configuration stub:
+
+        >>> provideAdapter(ApplicationStub(), (None,), ISchoolToolApplication)
+
+        >>> from schooltool.timetable.model import BaseTimetableModel
+        >>> btm = BaseTimetableModel()
+
+    We have a stub term that lasts from 2003-11-20 to 2003-11-26, and a simple
+    timetable.
+
+        >>> term = TermStub()
+        >>> timetable = createSimpleTimetable()
+        >>> from schooltool.timetable.interfaces import ITimetable
+        >>> provideAdapter(TimetablePhysicallyLocatableAdapterStub,
+        ...                (ITimetable,), IPhysicallyLocatable)
+
+    BaseTimetableModel is a base class and needs concrete implementations for
+    its abstract methods.
+
+        >>> btm._dayGenerator = lambda: itertools.cycle(['A', 'B'])
+        >>> btm.schooldayStrategy = lambda date, generator: generator.next()
+        >>> from schooltool.timetable import SchooldayTemplate, SchooldaySlot
+        >>> day_template = SchooldayTemplate()
+        >>> t, td = time, timedelta
+        >>> day_template.add(SchooldaySlot(t(9, 0), td(minutes=90)))
+        >>> day_template.add(SchooldaySlot(t(11, 05), td(minutes=80)))
+        >>> btm._getUsualTemplateForDay = lambda date, day_id: day_template
+
+    Let us create a calendar.
+
+        >>> from schooltool.timetable.interfaces import ITimetableCalendarEvent
+        >>> cal = btm.createCalendar(term, timetable,
+        ...                          first=date(2003, 11, 21),
+        ...                          last=date(2003, 11, 25))
+
+        >>> def print_cal(cal):
+        ...     for e in cal:
+        ...         print '%s %s--%s %s %-10s %s' % (
+        ...                     e.dtstart.date(), e.dtstart.strftime('%H:%M'),
+        ...                     (e.dtstart + e.duration).strftime('%H:%M'),
+        ...                     e.dtstart.tzinfo,
+        ...                     '(%s, %s)' % (e.day_id, e.period_id), e.title)
+        ...         assert verifyObject(ITimetableCalendarEvent, e)
+        ...         assert e.title == e.activity.title
+        >>> print_cal(cal)
+        2003-11-21 09:00--10:30 UTC (B, Green) Biology
+        2003-11-21 11:05--12:25 UTC (B, Blue)  Geography
+        2003-11-24 09:00--10:30 UTC (A, Green) English
+        2003-11-24 11:05--12:25 UTC (A, Blue)  Math
+        2003-11-25 09:00--10:30 UTC (B, Green) Biology
+        2003-11-25 11:05--12:25 UTC (B, Blue)  Geography
+
+    If school will get transported to Lithuania, lessons should still
+    start at the same local time (9:00 in the morning), but the time
+    should be stored in UTC:
+
+        >>> timetable.timezone = 'Europe/Vilnius'
+        >>> cal = btm.createCalendar(term, timetable,
+        ...                          first=date(2003, 11, 21),
+        ...                          last=date(2003, 11, 25))
+        >>> print_cal(cal)
+        2003-11-21 07:00--08:30 UTC (B, Green) Biology
+        2003-11-21 09:05--10:25 UTC (B, Blue)  Geography
+        2003-11-24 07:00--08:30 UTC (A, Green) English
+        2003-11-24 09:05--10:25 UTC (A, Blue)  Math
+        2003-11-25 07:00--08:30 UTC (B, Green) Biology
+        2003-11-25 09:05--10:25 UTC (B, Blue)  Geography
+
+    """
 
 
 class TestSequentialDaysTimetableModel(PlacelessSetup,
@@ -92,12 +180,12 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
                                        SequentialTestSetupMixin):
 
     def setUp(self):
-        from zope.app.traversing.interfaces import IPhysicallyLocatable
         from schooltool.timetable.interfaces import ITimetable
         PlacelessSetup.setUp(self)
 
-        ztapi.provideAdapter(ITimetable, IPhysicallyLocatable,
-                             TimetablePhysicallyLocatableAdapterStub)
+        provideAdapter(TimetablePhysicallyLocatableAdapterStub,
+                       (ITimetable,), IPhysicallyLocatable)
+        provideAdapter(ApplicationStub(), (None,), ISchoolToolApplication)
 
     def createModel(self):
         """Create a simple sequential timetable model.
@@ -108,16 +196,16 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
         Blue period occurs at 11:00-12:30 on all days except Fridays, when it
         occurs at 10:30-12:00.
         """
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
-        from schooltool.timetable import SequentialDaysTimetableModel
+        from schooltool.timetable import SchooldayTemplate, SchooldaySlot
+        from schooltool.timetable.model import SequentialDaysTimetableModel
 
         t, td = time, timedelta
         template1 = SchooldayTemplate()
-        template1.add(SchooldayPeriod('Green', t(9, 0), td(minutes=90)))
-        template1.add(SchooldayPeriod('Blue', t(11, 0), td(minutes=90)))
+        template1.add(SchooldaySlot(t(9, 0), td(minutes=90)))
+        template1.add(SchooldaySlot(t(11, 0), td(minutes=90)))
         template2 = SchooldayTemplate()
-        template2.add(SchooldayPeriod('Green', t(9, 0), td(minutes=90)))
-        template2.add(SchooldayPeriod('Blue', t(10, 30), td(minutes=90)))
+        template2.add(SchooldaySlot(t(9, 0), td(minutes=90)))
+        template2.add(SchooldaySlot(t(10, 30), td(minutes=90)))
 
         model = SequentialDaysTimetableModel(("A", "B"),
                                              {None: template1,
@@ -125,15 +213,15 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
         return model
 
     def test_interface(self):
-        from schooltool.timetable import SequentialDaysTimetableModel
+        from schooltool.timetable.model import SequentialDaysTimetableModel
         from schooltool.timetable.interfaces import IWeekdayBasedTimetableModel
 
         model = SequentialDaysTimetableModel(("A","B"), {None: 3})
         verifyObject(IWeekdayBasedTimetableModel, model)
 
     def test_eq(self):
-        from schooltool.timetable import SequentialDaysTimetableModel
-        from schooltool.timetable import WeeklyTimetableModel
+        from schooltool.timetable.model import SequentialDaysTimetableModel
+        from schooltool.timetable.model import WeeklyTimetableModel
         model = SequentialDaysTimetableModel(("A","B"), {1: 2, None: 3})
         model2 = SequentialDaysTimetableModel(("A","B"), {1: 2, None: 3})
         model3 = WeeklyTimetableModel(("A","B"), {1: 2, None: 3})
@@ -153,7 +241,7 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
         self.assertNotEqual(model, model2)
 
     def test_createCalendar(self):
-        from schoolbell.calendar.interfaces import ICalendar
+        from schooltool.calendar.interfaces import ICalendar
 
         tt = self.createTimetable()
         model = self.createModel()
@@ -186,8 +274,8 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
                          diff(pformat(expected), pformat(result)))
 
     def test_createCalendar_exceptionDays(self):
-        from schooltool.timetable import Term
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.term.term import Term
+        from schooltool.timetable import SchooldaySlot
 
         tt = self.createTimetable()
         model = self.createModel()
@@ -195,10 +283,9 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
         schooldays.addWeekdays(0, 1, 2, 3, 4, 5) # Mon-Sat
 
         # Add an exception day
-        exception = SchooldayTemplate()
         t, td = time, timedelta
-        exception.add(SchooldayPeriod('Green', t(6, 0), td(minutes=90)))
-        exception.add(SchooldayPeriod('Blue', t(8, 0), td(minutes=90)))
+        exception = [("Green", SchooldaySlot(t(6, 0), td(minutes=90))),
+                     ("Blue", SchooldaySlot(t(8, 0), td(minutes=90)))]
         model.exceptionDays[date(2003, 11, 22)] = exception
 
         # Run the calendar generation
@@ -224,7 +311,8 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
                          diff(pformat(expected), pformat(result)))
 
     def test_schooldayStrategy_getDayId(self):
-        from schooltool.timetable import SequentialDaysTimetableModel, Term
+        from schooltool.timetable.model import SequentialDaysTimetableModel
+        from schooltool.term.term import Term
         from schooltool.timetable import SchooldayTemplate
 
         term = Term('Sample', date(2005, 6, 27), date(2005, 7, 10))
@@ -259,35 +347,33 @@ class TestSequentialDaysTimetableModel(PlacelessSetup,
 
 
     def test_periodsInDay_originalPeriodsInDay(self):
-        from schoolbell.calendar.interfaces import ICalendar
-        from schooltool.timetable import SchooldayPeriod
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.timetable import SchooldaySlot
 
         tt = self.createTimetable()
         model = self.createModel()
         schooldays = TermStub()
 
         # Add an exception day
-        exception = SchooldayTemplate()
         t, td = time, timedelta
-        exception.add(SchooldayPeriod('Green', t(6, 0), td(minutes=90)))
-        exception.add(SchooldayPeriod('Blue', t(8, 0), td(minutes=90)))
+        exception = [
+            ("Green", SchooldaySlot(t(6, 0), td(minutes=90))),
+            ("Blue", SchooldaySlot(t(8, 0), td(minutes=90)))]
         model.exceptionDays[date(2003, 11, 21)] = exception
 
         self.assertEqual(
             model.periodsInDay(schooldays, tt, date(2003, 11, 20)),
-            [SchooldayPeriod("Green", time(9, 0), timedelta(minutes=90)),
-             SchooldayPeriod("Blue", time(11, 0), timedelta(minutes=90))])
+            [('Green', time(9, 0), timedelta(minutes=90)),
+             ('Blue',  time(11, 0), timedelta(minutes=90))])
 
         self.assertEqual(
             model.originalPeriodsInDay(schooldays, tt, date(2003, 11, 21)),
-            [SchooldayPeriod("Green", time(9, 0), timedelta(minutes=90)),
-             SchooldayPeriod("Blue", time(10, 30), timedelta(minutes=90))])
+            [('Green', time(9, 0), timedelta(minutes=90)),
+             ('Blue', time(10, 30), timedelta(minutes=90))])
 
         self.assertEqual(
             model.periodsInDay(schooldays, tt, date(2003, 11, 21)),
-            [SchooldayPeriod("Green", time(6, 0), timedelta(minutes=90)),
-             SchooldayPeriod("Blue", time(8, 0), timedelta(minutes=90))])
+            [('Green', time(6, 0), timedelta(minutes=90)),
+             ('Blue', time(8, 0), timedelta(minutes=90))])
 
         self.assertEqual(
             model.periodsInDay(schooldays, tt, date(2003, 11, 22)),
@@ -300,27 +386,27 @@ class TestSequentialDayIdBasedTimetableModel(PlacelessSetup,
                                              BaseTestTimetableModel):
 
     def setUp(self):
-        from zope.app.traversing.interfaces import IPhysicallyLocatable
+        from zope.traversing.interfaces import IPhysicallyLocatable
         from schooltool.timetable.interfaces import ITimetable
         PlacelessSetup.setUp(self)
 
-        ztapi.provideAdapter(ITimetable, IPhysicallyLocatable,
-                             TimetablePhysicallyLocatableAdapterStub)
+        provideAdapter(TimetablePhysicallyLocatableAdapterStub,
+                       (ITimetable,), IPhysicallyLocatable)
+        provideAdapter(ApplicationStub(), (None,), ISchoolToolApplication)
 
     def createDayTemplates(self):
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.timetable import SchooldayTemplate, SchooldaySlot
         t, td = time, timedelta
         template1 = SchooldayTemplate()
-        template1.add(SchooldayPeriod('Green', t(9, 0), td(minutes=90)))
-        template1.add(SchooldayPeriod('Blue', t(11, 0), td(minutes=90)))
+        template1.add(SchooldaySlot(t(9, 0), td(minutes=90)))
+        template1.add(SchooldaySlot(t(11, 0), td(minutes=90)))
         template2 = SchooldayTemplate()
-        template2.add(SchooldayPeriod('Green', t(11, 0), td(minutes=90)))
-        template2.add(SchooldayPeriod('Blue', t(13, 0), td(minutes=90)))
+        template2.add(SchooldaySlot(t(11, 0), td(minutes=90)))
+        template2.add(SchooldaySlot(t(13, 0), td(minutes=90)))
         return template1, template2
 
     def test_createCalendar(self):
-        from schoolbell.calendar.interfaces import ICalendar
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.calendar.interfaces import ICalendar
         from schooltool.timetable.model import \
              SequentialDayIdBasedTimetableModel
 
@@ -358,6 +444,30 @@ class TestSequentialDayIdBasedTimetableModel(PlacelessSetup,
         self.assertEqual(expected, result,
                          diff(pformat(expected), pformat(result)))
 
+    def test_createCalendar_date_range(self):
+        from schooltool.timetable.model \
+                import SequentialDayIdBasedTimetableModel
+        template1, template2 = self.createDayTemplates()
+        model = SequentialDayIdBasedTimetableModel(('A', 'B'),
+                                                   {'A': template1,
+                                                    'B': template2})
+        schooldays = TermStub()
+        tt = self.createTimetable()
+        cal = model.createCalendar(schooldays, tt, first=date(2003, 11, 21),
+                                   last=date(2003, 11, 25))
+        result = self.extractCalendarEvents(cal, schooldays)
+        expected = [{},
+                    {datetime(2003, 11, 21, 11, 0, tzinfo=UTC): "Biology",
+                     datetime(2003, 11, 21, 13, 0, tzinfo=UTC): "Geography"},
+                    {}, {},
+                    {datetime(2003, 11, 24, 9, 0, tzinfo=UTC): "English",
+                     datetime(2003, 11, 24, 11, 0, tzinfo=UTC): "Math"},
+                    {datetime(2003, 11, 25, 11, 0, tzinfo=UTC): "Biology",
+                     datetime(2003, 11, 25, 13, 0, tzinfo=UTC): "Geography"},
+                    {}]
+        self.assertEqual(expected, result,
+                         diff(pformat(expected), pformat(result)))
+
     def test_verification(self):
         from schooltool.timetable.model import \
              SequentialDayIdBasedTimetableModel
@@ -396,18 +506,19 @@ class TestWeeklyTimetableModel(PlacelessSetup,
                                BaseTestTimetableModel):
 
     def setUp(self):
-        from zope.app.traversing.interfaces import IPhysicallyLocatable
+        from zope.traversing.interfaces import IPhysicallyLocatable
         from schooltool.timetable.interfaces import ITimetable
         PlacelessSetup.setUp(self)
-        ztapi.provideAdapter(ITimetable, IPhysicallyLocatable,
-                             TimetablePhysicallyLocatableAdapterStub)
+        provideAdapter(TimetablePhysicallyLocatableAdapterStub,
+                       (ITimetable,), IPhysicallyLocatable)
+        provideAdapter(ApplicationStub(), (None,), ISchoolToolApplication)
 
     def test(self):
-        from schooltool.timetable import WeeklyTimetableModel
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.timetable.model import WeeklyTimetableModel
+        from schooltool.timetable import SchooldayTemplate, SchooldaySlot
         from schooltool.timetable import Timetable, TimetableDay
         from schooltool.timetable import TimetableActivity
-        from schooltool.timetable import Term
+        from schooltool.term.term import Term
         from schooltool.timetable.interfaces import IWeekdayBasedTimetableModel
 
         days = ('Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday')
@@ -443,21 +554,21 @@ class TestWeeklyTimetableModel(PlacelessSetup,
 
         t, td = time, timedelta
         template = SchooldayTemplate()
-        template.add(SchooldayPeriod('1', t(9, 0), td(minutes=45)))
-        template.add(SchooldayPeriod('2', t(9, 50), td(minutes=45)))
-        template.add(SchooldayPeriod('3', t(10, 50), td(minutes=45)))
-        template.add(SchooldayPeriod('4', t(12, 0), td(minutes=45)))
+        template.add(SchooldaySlot(t(9, 0), td(minutes=45)))
+        template.add(SchooldaySlot(t(9, 50), td(minutes=45)))
+        template.add(SchooldaySlot(t(10, 50), td(minutes=45)))
+        template.add(SchooldaySlot(t(12, 0), td(minutes=45)))
 
         model = WeeklyTimetableModel(day_templates={None: template})
         verifyObject(IWeekdayBasedTimetableModel, model)
 
         # Add an exception day
-        exception = SchooldayTemplate()
         t, td = time, timedelta
-        exception.add(SchooldayPeriod('1', t(6, 0), td(minutes=45)))
-        exception.add(SchooldayPeriod('2', t(7, 0), td(minutes=45)))
-        exception.add(SchooldayPeriod('3', t(8, 0), td(minutes=45)))
-        exception.add(SchooldayPeriod('4', t(9, 0), td(minutes=45)))
+        exception = [
+            ('1', SchooldaySlot(t(6, 0), td(minutes=45))),
+            ('2', SchooldaySlot(t(7, 0), td(minutes=45))),
+            ('3', SchooldaySlot(t(8, 0), td(minutes=45))),
+            ('4', SchooldaySlot(t(9, 0), td(minutes=45)))]
         model.exceptionDays[date(2003, 11, 22)] = exception
         model.exceptionDayIds[date(2003, 11, 22)] = "Monday"
 
@@ -501,11 +612,11 @@ class TestWeeklyTimetableModel(PlacelessSetup,
                          diff(pformat(expected), pformat(result)))
 
     def test_not_enough_days(self):
-        from schooltool.timetable import WeeklyTimetableModel
-        from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
+        from schooltool.timetable.model import WeeklyTimetableModel
+        from schooltool.timetable import SchooldayTemplate, SchooldaySlot
         from schooltool.timetable import Timetable, TimetableDay
         template = SchooldayTemplate()
-        template.add(SchooldayPeriod('1', time(8), timedelta(minutes=30)))
+        template.add(SchooldaySlot(time(8), timedelta(minutes=30)))
         days = ["Mon", "Tue"]
         model = WeeklyTimetableModel(days, {None: template})
         day = date(2003, 11, 20)    # 2003-11-20 is a Thursday
@@ -520,7 +631,8 @@ class TestWeeklyTimetableModel(PlacelessSetup,
         model.createCalendar(schooldays, tt)
 
     def test_schooldayStrategy(self):
-        from schooltool.timetable import WeeklyTimetableModel, Term
+        from schooltool.timetable.model import WeeklyTimetableModel
+        from schooltool.term.term import Term
         from schooltool.timetable import SchooldayTemplate
 
         term = Term('Sample', date(2005, 6, 27), date(2005, 7, 10))
@@ -553,27 +665,193 @@ class TestWeeklyTimetableModel(PlacelessSetup,
 class TestTimetableCalendarEvent(unittest.TestCase):
 
     def test(self):
-        from schooltool.timetable import TimetableCalendarEvent
+        from schooltool.timetable.model import TimetableCalendarEvent
         from schooltool.timetable.interfaces import ITimetableCalendarEvent
 
+        day_id = 'Day2'
         period_id = 'Mathematics'
-        activity = object()
+
+        class ActivityStub(object):
+            owner = None
+            resources = []
+        activity = ActivityStub()
 
         ev = TimetableCalendarEvent(datetime(2004, 10, 13, 12),
                                     timedelta(45), "Math",
-                                    period_id=period_id, activity=activity)
+                                    day_id=day_id, period_id=period_id,
+                                    activity=activity)
         verifyObject(ITimetableCalendarEvent, ev)
-        for attr in ['period_id', 'activity']:
+        for attr in ['day_id', 'period_id', 'activity']:
             self.assertRaises(AttributeError, setattr, ev, attr, object())
 
+    def test_owner_resources(self):
+        from schooltool.timetable.model import TimetableCalendarEvent
+
+        day_id = 'Day2'
+        period_id = 'Mathematics'
+
+        class ActivityStub(object):
+            resources = ['cookie', 'parrot']
+            owner = ['Long John']
+        activity = ActivityStub()
+
+        ev = TimetableCalendarEvent(datetime(2004, 10, 13, 12),
+                                    timedelta(45), "Sailin mate",
+                                    day_id=day_id, period_id=period_id,
+                                    activity=activity)
+
+        self.assertEqual(ev.owner, activity.owner)
+        self.assertEqual(ev.resources, activity.resources)
+
+
+def doctest_PersistentTimetableCalendarEvent():
+    """Tests for PersistentTimetableCalendarEvent.
+
+        >>> from schooltool.timetable.model import PersistentTimetableCalendarEvent
+        >>> from schooltool.timetable.model import TimetableCalendarEvent
+        >>> class TTStub(object):
+        ...     timezone = 'utc'
+        >>> class ActivityStub(object):
+        ...     resources = ['cookie', 'parrot']
+        ...     owner = ['Long John']
+        ...     timetable = TTStub()
+        >>> activity = ActivityStub()
+        >>> ev = TimetableCalendarEvent(datetime(2004, 10, 13, 23, 0),
+        ...                             timedelta(45), "Math",
+        ...                             day_id='Day 2', period_id='Lesson 1',
+        ...                             activity=activity)
+        >>> class PTCEStub(PersistentTimetableCalendarEvent):
+        ...     def bookResource(self, resource):
+        ...         print "Booking %s" % resource
+        >>> event = PTCEStub(ev)
+        Booking cookie
+        Booking parrot
+
+        >>> event.schoolDay()
+        datetime.date(2004, 10, 13)
+
+        >>> event.activity.timetable.timezone = 'Europe/Vilnius'
+        >>> event.schoolDay()
+        datetime.date(2004, 10, 14)
+
+    """
+
+def doctest_timetableEventHandlers():
+    """
+        >>> app = ApplicationStub()
+        >>> provideAdapter(app, (None,), ISchoolToolApplication)
+        >>> timetable_calendar = []
+        >>> class ModelStub(object):
+        ...     def createCalendar(self, term, timetable, first=None, last=None):
+        ...         print "Creating calendar for %s" % term
+        ...         return timetable_calendar
+        >>> class TimetableStub(object):
+        ...     __name__ = 'term.schema'
+        ...     term = 'Term for Fall 2006'
+        ...     model = ModelStub()
+        >>> from schooltool.app.interfaces import ISchoolToolCalendar
+        >>> class CalendarStub(list):
+        ...     implements(ISchoolToolCalendar)
+        ...     def addEvent(self, event):
+        ...         print "Adding event %s to a calendar" % event.__name__
+        ...     def removeEvent(self, event):
+        ...         print "Removing event %s to a calendar" % event.__name__
+
+        >>> section_calendar = CalendarStub()
+        >>> class SectionStub(object):
+        ...     def __conform__(self, iface):
+        ...         if iface == ISchoolToolCalendar:
+        ...             return section_calendar
+        >>> class ActivityStub(object):
+        ...     timetable = TimetableStub()
+        ...     owner = SectionStub()
+        ...     def __eq__(self, other):
+        ...         return isinstance(other, ActivityStub)
+        >>> class EventStub(object):
+        ...     activity = ActivityStub()
+        ...     period_id = 'Period 1'
+        ...     day_id = 'Day 1'
+        >>> from schooltool.timetable.model import addEventsToCalendar
+        >>> event = EventStub()
+        >>> addEventsToCalendar(event)
+        Creating calendar for Term for Fall 2006
+
+        >>> from schooltool.timetable.interfaces import ITimetableCalendarEvent
+        >>> class TimetableEventStub(object):
+        ...     implements(ITimetableCalendarEvent)
+        ...     def __init__(self, name, activity, period_id, day_id):
+        ...         self.activity = activity
+        ...         self.period_id = period_id
+        ...         self.day_id = day_id
+        ...         self.unique_id = 'event'
+        ...         self.__name__ = name
+        ...         self.dtstart = None
+        ...         self.duration = None
+        ...         self.description = None
+        ...         self.location = None
+        ...         self.recurrence = None
+        ...         self.allday = None
+        ...         self.resources = []
+        >>> timetable_calendar = [
+        ...     TimetableEventStub('TTEvent1', ActivityStub(), 'Period 1', 'Day 1'),
+        ...     TimetableEventStub('TTEvent2', ActivityStub(), 'Period 1', 'Day 2'),
+        ...     TimetableEventStub('TTEvent3', ActivityStub(), 'Period 2', 'Day 2'),
+        ...     TimetableEventStub('TTEvent4', object(), 'Period 2', 'Day 2')]
+        >>> addEventsToCalendar(event)
+        Creating calendar for Term for Fall 2006
+        Adding event TTEvent1 to a calendar
+
+        >>> section_calendar[:] = [
+        ...     TimetableEventStub('TTEvent1', ActivityStub(), 'Period 1', 'Day 1'),
+        ...     TimetableEventStub('TTEvent2', ActivityStub(), 'Period 1', 'Day 2'),
+        ...     TimetableEventStub('TTEvent3', ActivityStub(), 'Period 2', 'Day 2'),
+        ...     TimetableEventStub('TTEvent4', ActivityStub(), 'Period 2', 'Day 2')]
+        >>> from schooltool.timetable.model import removeEventsFromCalendar
+        >>> removeEventsFromCalendar(event)
+        Removing event TTEvent1 to a calendar
+
+        >>> from schooltool.timetable import TimetableReplacedEvent
+        >>> from schooltool.timetable.model import handleTimetableRemovedEvent
+        >>> section = SectionStub()
+        >>> key = "term.schema"
+        >>> event = TimetableReplacedEvent(section, key,
+        ...                                old_timetable=ActivityStub.timetable,
+        ...                                new_timetable=None)
+        >>> handleTimetableRemovedEvent(event)
+        Removing event TTEvent1 to a calendar
+        Removing event TTEvent2 to a calendar
+        Removing event TTEvent3 to a calendar
+        Removing event TTEvent4 to a calendar
+
+        >>> event = TimetableReplacedEvent(section, key,
+        ...                                old_timetable=None,
+        ...                                new_timetable=None)
+        >>> handleTimetableRemovedEvent(event)
+
+        >>> from schooltool.timetable.model import handleTimetableAddedEvent
+        >>> event = TimetableReplacedEvent(section, key,
+        ...                                old_timetable=None,
+        ...                                new_timetable=ActivityStub.timetable)
+        >>> handleTimetableAddedEvent(event)
+        Creating calendar for Term for Fall 2006
+        Adding event TTEvent1 to a calendar
+        Adding event TTEvent2 to a calendar
+        Adding event TTEvent3 to a calendar
+        Adding event TTEvent4 to a calendar
+
+    """
+
 
 def test_suite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TestSequentialDaysTimetableModel))
-    suite.addTest(unittest.makeSuite(TestSequentialDayIdBasedTimetableModel))
-    suite.addTest(unittest.makeSuite(TestWeeklyTimetableModel))
-    suite.addTest(unittest.makeSuite(TestTimetableCalendarEvent))
-    return suite
+    return unittest.TestSuite([
+        doctest.DocTestSuite(setUp=setup.placelessSetUp,
+                             tearDown=setup.placelessTearDown),
+        unittest.makeSuite(TestSequentialDaysTimetableModel),
+        unittest.makeSuite(TestSequentialDayIdBasedTimetableModel),
+        unittest.makeSuite(TestWeeklyTimetableModel),
+        unittest.makeSuite(TestTimetableCalendarEvent),
+    ])
 
 if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
+
